@@ -13,7 +13,7 @@ import itertools
 import urlparse
 
 from django.http import QueryDict
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import ugettext as _
 
 from sentry.app import env
 from sentry.models import UserOption
@@ -92,8 +92,11 @@ class Interface(object):
     def to_string(self, event):
         return ''
 
+    def get_slug(self):
+        return type(self).__name__.lower()
+
     def get_title(self):
-        return _(self.__class__.__name__)
+        return _(type(self).__name__)
 
     def get_search_context(self, event):
         """
@@ -144,7 +147,7 @@ class Message(Interface):
         elif isinstance(self.params, dict):
             params = self.params.values()
         else:
-            params = ()
+            params = []
         return {
             'text': [self.message] + params,
         }
@@ -189,6 +192,8 @@ class Stacktrace(Interface):
     hash must contain **at least** the ``filename`` attribute. The rest of the values
     are optional, but recommended.
 
+    The list of frames should be ordered by the oldest call first.
+
     Each frame must contain the following attributes:
 
     ``filename``
@@ -254,19 +259,6 @@ class Stacktrace(Interface):
             if 'in_app' in frame:
                 frame['in_app'] = bool(frame['in_app'])
 
-    def _shorten(self, value, depth=1):
-        if depth > 5:
-            return type(value)
-        if isinstance(value, dict):
-            return dict((k, self._shorten(v, depth + 1)) for k, v in sorted(value.iteritems())[:100 / depth])
-        elif isinstance(value, (list, tuple, set, frozenset)):
-            return tuple(self._shorten(v, depth + 1) for v in value)[:100 / depth]
-        elif isinstance(value, (int, long, float)):
-            return value
-        elif not value:
-            return value
-        return value
-
     def serialize(self):
         return {
             'frames': self.frames,
@@ -281,19 +273,41 @@ class Stacktrace(Interface):
     def get_hash(self):
         output = []
         for frame in self.frames:
-            if frame.get('module'):
-                output.append(frame['module'])
-            else:
-                output.append(frame['filename'])
-
-            if frame.get('context_line'):
-                output.append(frame['context_line'])
-            elif frame.get('function'):
-                output.append(frame['function'])
-            elif frame.get('lineno'):
-                output.append(frame['lineno'])
+            output.extend(self.get_frame_hash(frame))
         return output
 
+    def get_frame_hash(self, frame):
+        output = []
+        if frame.get('module'):
+            output.append(frame['module'])
+        else:
+            output.append(frame['filename'])
+
+        if frame.get('context_line'):
+            output.append(frame['context_line'])
+        elif frame.get('function'):
+            output.append(frame['function'])
+        elif frame.get('lineno'):
+            output.append(frame['lineno'])
+        return output
+
+    def is_newest_frame_first(self, event):
+        newest_first = event.platform not in ('python', None)
+
+        if env.request and env.request.user.is_authenticated():
+            display = UserOption.objects.get_value(
+                user=env.request.user,
+                project=None,
+                key='stacktrace_order',
+                default=None,
+            )
+            if display == '1':
+                newest_first = False
+            elif display == '2':
+                newest_first = True
+
+        return newest_first
+
     def to_html(self, event):
         system_frames = 0
         frames = []
@@ -307,7 +321,7 @@ class Stacktrace(Interface):
 
             context_vars = []
             if frame.get('vars'):
-                context_vars = self._shorten(frame['vars'])
+                context_vars = frame['vars']
             else:
                 context_vars = []
 
@@ -335,31 +349,43 @@ class Stacktrace(Interface):
         if len(frames) == system_frames:
             system_frames = 0
 
-        if env.request and env.request.user.is_authenticated():
-            display = UserOption.objects.get_value(
-                user=env.request.user,
-                project=None,
-                key='stacktrace_order',
-                default=None,
-            )
-            if display == '2':
-                frames.reverse()
+        newest_first = self.is_newest_frame_first(event)
+        if newest_first:
+            frames = frames[::-1]
 
         return render_to_string('sentry/partial/interfaces/stacktrace.html', {
+            'newest_first': newest_first,
             'system_frames': system_frames,
             'event': event,
             'frames': frames,
-            'stacktrace': self.get_traceback(event),
+            'stacktrace': self.get_traceback(event, newest_first=newest_first),
         })
 
     def to_string(self, event):
-        return self.get_stacktrace(event)
+        return self.get_stacktrace(event, system_frames=False)
 
-    def get_stacktrace(self, event):
-        result = [
-            'Stacktrace (most recent call last):', '',
-        ]
-        for frame in self.frames:
+    def get_stacktrace(self, event, system_frames=True, newest_first=None):
+        if newest_first is None:
+            newest_first = self.is_newest_frame_first(event)
+
+        result = []
+        if newest_first:
+            result.append(_('Stacktrace (most recent call first):'))
+        else:
+            result.append(_('Stacktrace (most recent call last):'))
+
+        result.append('')
+
+        frames = self.frames
+        if not system_frames:
+            frames = [f for f in frames if f.get('in_app')]
+            if not frames:
+                frames = self.frames
+
+        if newest_first:
+            frames = frames[::-1]
+
+        for frame in frames:
             pieces = ['  File "%(filename)s"']
             if 'lineno' in frame:
                 pieces.append(', line %(lineno)s')
@@ -372,10 +398,10 @@ class Stacktrace(Interface):
 
         return '\n'.join(result)
 
-    def get_traceback(self, event):
+    def get_traceback(self, event, newest_first=None):
         result = [
             event.message, '',
-            self.get_stacktrace(event),
+            self.get_stacktrace(event, newest_first=newest_first),
         ]
 
         return '\n'.join(result)
@@ -564,6 +590,9 @@ class Http(Interface):
             'env': self.env,
         })
 
+    def get_title(self):
+        return _('Request')
+
     def get_search_context(self, event):
         return {
             'filters': {
