@@ -14,6 +14,7 @@ import time
 import uuid
 import urlparse
 
+from datetime import timedelta
 from hashlib import md5
 from indexer.models import BaseIndex
 from picklefield.fields import PickledObjectField
@@ -233,6 +234,10 @@ class ProjectKey(Model):
     secret_key = models.CharField(max_length=32, unique=True, null=True)
     user = models.ForeignKey(User, null=True)
 
+    # For audits
+    user_added = models.ForeignKey(User, null=True, related_name='keys_added_set')
+    date_added = models.DateTimeField(default=timezone.now, null=True)
+
     objects = BaseManager(cache_fields=(
         'public_key',
         'secret_key',
@@ -268,6 +273,14 @@ class ProjectKey(Model):
             self.project_id,
         )
 
+    @property
+    def dsn_private(self):
+        return self.get_dsn(public=False)
+
+    @property
+    def dsn_public(self):
+        return self.get_dsn(public=True)
+
 
 class ProjectOption(Model):
     """
@@ -329,7 +342,7 @@ class PendingTeamMember(Model):
         body = render_to_string('sentry/emails/member_invite.txt', context)
 
         try:
-            send_mail('%s Invite to join team: %s' % (settings.EMAIL_SUBJECT_PREFIX, self.team.name),
+            send_mail('%sInvite to join team: %s' % (settings.EMAIL_SUBJECT_PREFIX, self.team.name),
                 body, settings.SERVER_EMAIL, [self.email],
                 fail_silently=False)
         except Exception, e:
@@ -348,6 +361,7 @@ class MessageBase(Model):
     culprit = models.CharField(max_length=200, blank=True, null=True, db_column='view')
     checksum = models.CharField(max_length=32, db_index=True)
     data = GzippedDictField(blank=True, null=True)
+    platform = models.CharField(max_length=64, null=True)
 
     class Meta:
         abstract = True
@@ -750,6 +764,46 @@ class UserOption(Model):
         return u'user=%s, project=%s, key=%s, value=%s' % (self.user_id, self.project_id, self.key, self.value)
 
 
+class LostPasswordHash(models.Model):
+    user = models.ForeignKey(User, unique=True)
+    hash = models.CharField(max_length=32)
+    date_added = models.DateTimeField(default=timezone.now)
+
+    def save(self, *args, **kwargs):
+        if not self.hash:
+            self.set_hash()
+        super(LostPasswordHash, self).save(*args, **kwargs)
+
+    def set_hash(self):
+        import hashlib
+        import random
+
+        self.hash = hashlib.md5(str(random.randint(1, 10000000))).hexdigest()
+
+    def is_valid(self):
+        return self.date_added > timezone.now() - timedelta(days=1)
+
+    def send_recover_mail(self):
+        from django.core.mail import send_mail
+        from sentry.web.helpers import render_to_string
+
+        context = {
+            'user': self.user,
+            'domain': urlparse.urlparse(settings.URL_PREFIX).hostname,
+            'url': '%s%s' % (settings.URL_PREFIX,
+                reverse('sentry-account-recover-confirm', args=[self.user.id, self.hash])),
+        }
+        body = render_to_string('sentry/emails/recover_account.txt', context)
+
+        try:
+            send_mail('%sPassword Recovery' % (settings.EMAIL_SUBJECT_PREFIX,),
+                body, settings.SERVER_EMAIL, [self.user.email],
+                fail_silently=False)
+        except Exception, e:
+            logger = logging.getLogger('sentry.mail.errors')
+            logger.exception(e)
+
+
 ### django-indexer
 
 
