@@ -33,8 +33,8 @@ class SentryHTTPServer(Service):
         #     worker.init_process = profiling_init_process.__get__(worker)
 
         options = (settings.WEB_OPTIONS or {}).copy()
-        options['bind'] = '%s:%s' % (self.host, self.port)
         options['debug'] = debug
+        options.setdefault('bind', '%s:%s' % (self.host, self.port))
         options.setdefault('daemon', False)
         options.setdefault('timeout', 30)
         if workers:
