@@ -5,15 +5,17 @@ sentry.web.frontend.projects
 :copyright: (c) 2012 by the Sentry Team, see AUTHORS for more details.
 :license: BSD, see LICENSE for more details.
 """
+from django.contrib import messages
 from django.core.context_processors import csrf
 from django.core.urlresolvers import reverse
 from django.http import HttpResponseRedirect
 from django.views.decorators.csrf import csrf_protect
+from django.views.decorators.http import require_http_methods
 
-from sentry.constants import MEMBER_OWNER, MEMBER_USER
-from sentry.models import TeamMember, ProjectKey, Team, FilterKey
+from sentry.constants import MEMBER_OWNER
+from sentry.models import TeamMember, ProjectKey, Team, FilterKey, Group
 from sentry.permissions import can_create_projects, can_remove_project, can_create_teams, \
-  can_add_team_member
+  can_add_team_member, can_add_project_key, can_remove_project_key
 from sentry.plugins import plugins
 from sentry.plugins.helpers import set_option, get_option
 from sentry.web.decorators import login_required, has_access
@@ -24,10 +26,26 @@ from sentry.web.helpers import render_to_response, get_project_list, \
   plugin_config
 
 
+@login_required
+@has_access
+def dashboard(request, project):
+    if not Group.objects.filter(project=project).exists():
+        return HttpResponseRedirect(reverse('sentry-get-started', args=[project.slug]))
+    return HttpResponseRedirect(reverse('sentry-stream', args=[project.slug]))
+
+
+@login_required
+@has_access
+def get_started(request, project):
+    return render_to_response('sentry/get_started.html', {
+        'project': project,
+    }, request)
+
+
 @login_required
 def project_list(request):
     project_list = get_project_list(request.user, hidden=True).values()
-    team_list = dict((t.id, t) for t in Team.objects.filter(pk__in=[p.team_id for p in project_list]))
+    team_list = Team.objects.in_bulk([p.team_id for p in project_list])
     if request.user.is_authenticated():
         memberships = dict((tm.team_id, tm) for tm in TeamMember.objects.filter(user=request.user, team__in=team_list))
         keys = dict((p.project_id, p) for p in ProjectKey.objects.filter(user=request.user, project__in=project_list))
@@ -92,15 +110,18 @@ def new_project(request):
         project = project_form.save(commit=False)
         if not project.owner:
             project.owner = request.user
+
         if is_new_team:
             team = new_team_form.save(commit=False)
             team.owner = project.owner
             team.save()
         else:
             team = select_team_form.cleaned_data['team']
+
         project.team = team
         project.save()
-        return HttpResponseRedirect(reverse('sentry-project-client-help', args=[project.slug]))
+
+        return HttpResponseRedirect(reverse('sentry-get-started', args=[project.slug]))
 
     return render_to_response('sentry/projects/new.html', {
         'project_form': project_form,
@@ -174,6 +195,7 @@ def manage_project(request, project):
         'form': form,
         'project': project,
         'TEAM_LIST': team_list.values(),
+        'SECTION': 'settings',
     })
 
     return render_to_response('sentry/projects/manage.html', context, request)
@@ -186,42 +208,86 @@ def manage_project_team(request, project):
     if result is False and not request.user.has_perm('sentry.can_change_project'):
         return HttpResponseRedirect(reverse('sentry'))
 
-    if not project.team:
+    team = project.team
+
+    if not team:
         member_list = []
+        pending_member_list = []
     else:
-        member_list = [(tm, tm.user) for tm in project.team.member_set.select_related('user')]
+        member_list = [(tm, tm.user) for tm in team.member_set.select_related('user')]
+        pending_member_list = [(pm, pm.email) for pm in team.pending_member_set.all().order_by('email')]
 
     context = csrf(request)
     context.update({
         'page': 'team',
         'project': project,
-        'team': project.team,
+        'team': team,
         'member_list': member_list,
+        'pending_member_list': pending_member_list,
         'can_add_member': can_add_team_member(request.user, project.team),
+        'SECTION': 'settings',
     })
 
     return render_to_response('sentry/projects/team.html', context, request)
 
 
-@has_access(MEMBER_USER)
-def client_help(request, project):
-    try:
-        key = ProjectKey.objects.get(user=request.user, project=project)
-    except ProjectKey.DoesNotExist:
-        key = None  # superuser
+@has_access(MEMBER_OWNER)
+@csrf_protect
+def manage_project_keys(request, project):
+    result = plugins.first('has_perm', request.user, 'edit_project', project)
+    if result is False and not request.user.has_perm('sentry.can_change_project'):
+        return HttpResponseRedirect(reverse('sentry'))
 
-    context = {
-        'can_remove_project': can_remove_project(request.user, project),
-        'page': 'client_help',
+    key_list = list(ProjectKey.objects.filter(
+        project=project,
+    ).select_related('user', 'user_added').order_by('-id'))
+
+    for key in key_list:
+        key.project = project
+        key.can_remove = can_remove_project_key(request.user, key),
+
+    context = csrf(request)
+    context.update({
+        'page': 'keys',
         'project': project,
-        'key': key,
-    }
-    if key:
-        context.update({
-            'dsn': key.get_dsn(),
-            'dsn_public': key.get_dsn(public=True),
-        })
-    return render_to_response('sentry/projects/client_help.html', context, request)
+        'key_list': key_list,
+        'can_add_key': can_add_project_key(request.user, project),
+        'SECTION': 'settings',
+    })
+
+    return render_to_response('sentry/projects/keys.html', context, request)
+
+
+@has_access(MEMBER_OWNER)
+@csrf_protect
+def new_project_key(request, project):
+    if not can_add_project_key(request.user, project):
+        return HttpResponseRedirect(reverse('sentry-manage-project-keys', args=[project.slug]))
+
+    ProjectKey.objects.create(
+        project=project,
+        user_added=request.user,
+    )
+
+    return HttpResponseRedirect(reverse('sentry-manage-project-keys', args=[project.slug]))
+
+
+@require_http_methods(['POST'])
+@has_access(MEMBER_OWNER)
+@csrf_protect
+def remove_project_key(request, project, key_id):
+    try:
+        key = ProjectKey.objects.get(id=key_id)
+    except ProjectKey.DoesNotExist:
+        return HttpResponseRedirect(reverse('sentry-manage-project-keys', args=[project.slug]))
+
+    if not can_remove_project_key(request.user, key):
+        return HttpResponseRedirect(reverse('sentry-manage-project-keys', args=[project.slug]))
+
+    key.delete()
+    messages.add_message(request, messages.SUCCESS, 'The API key (%s) was revoked.' % (key.public_key,))
+
+    return HttpResponseRedirect(reverse('sentry-manage-project-keys', args=[project.slug]))
 
 
 @has_access(MEMBER_OWNER)
@@ -241,6 +307,7 @@ def manage_project_tags(request, project):
         'page': 'tags',
         'project': project,
         'form': form,
+        'SECTION': 'settings',
     }
     return render_to_response('sentry/projects/manage_tags.html', context, request)
 
@@ -263,6 +330,7 @@ def manage_plugins(request, project):
     context.update({
         'page': 'plugins',
         'project': project,
+        'SECTION': 'settings',
     })
 
     return render_to_response('sentry/projects/plugins/list.html', context, request)
@@ -298,6 +366,7 @@ def configure_project_plugin(request, project, slug):
         'view': view,
         'project': project,
         'plugin': plugin,
+        'SECTION': 'settings',
     })
 
     return render_to_response('sentry/projects/plugins/configure.html', context, request)
